# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import asyncio
import concurrent
import functools
import logging

log = logging.getLogger(__name__)


class TaskBase(object):
    parallel_executors = 1

    def __init__(self):
        self.executors = concurrent.futures.ThreadPoolExecutor(max_workers=self.parallel_executors)

    async def _run_in_thread(self, func, *args, **kwargs):
        event_loop = asyncio.get_event_loop()
        res = await event_loop.run_in_executor(self.executors,
                                               functools.partial(func, *args, **kwargs))
        return res

    @staticmethod
    async def _run_command(*args):
        """
        Asynchronous run command in subprocess

        :param *args: command to execute
        :return: tuple (stdout, stderr, exit code)

        Example from: http://asyncio.readthedocs.io/en/latest/subprocess.html
        """
        # Create subprocess
        log.debug("Executing subprocess: %s", " ".join([a for a in args]))
        # pylint: disable=no-value-for-parameter
        process = await asyncio.create_subprocess_exec(
            *args, stderr=asyncio.subprocess.PIPE, stdout=asyncio.subprocess.PIPE)
        # pylint: enable=no-value-for-parameter

        # Status
        log.info('Started: %s (pid = %s)', args, process.pid)

        # Wait for the subprocess to finish
        stdout, stderr = await process.communicate()

        if process.returncode == 0:
            log.debug('Subprocess pid %s succeeded: %s', process.pid, args)
        else:
            log.debug('Subprocess pid %s failed: %s', process.pid, args)

        stdout_str = stdout.decode().strip()
        stderr_str = stderr.decode().strip()

        # Return stdout, stderr, exit code
        return stdout_str, stderr_str, process.returncode



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import logging
import os

from babelfish import Language
from subliminal import Video
from subliminal import download_best_subtitles
from subliminal import region
from subliminal import save_subtitles
from subliminal.subtitle import get_subtitle_path

from dopplerr.tasks.base import TaskBase

log = logging.getLogger(__name__)


class SubliminalTask(TaskBase):
    parallel_executors = 1

    @staticmethod
    def initialize_db():
        log.info("Initializing Subliminal cache...")
        region.configure('dogpile.cache.dbm', arguments={'filename': 'cachefile.dbm'})

    async def download_sub_by_subproc(self, videos, languages, provider_configs):
        subl_cmd = ["subliminal"]
        print(provider_configs)
        for provider_name, provider_config in provider_configs.items():
            subl_cmd.extend([
                "--{}".format(provider_name),
                provider_config['username'],
                provider_config['password'],
            ])
        subl_cmd.extend(["download"])
        for l in languages:
            subl_cmd.extend(["--language", l])
        subl_cmd.extend(videos)
        subl_cmd.extend(["-vvv"])
        stdout, stderr, code = await self._run_command(*subl_cmd)
        log.debug(stdout)
        log.error(stderr)
        log.error(code)
        if "Downloaded 0 subtitle" in stdout:
            log.error("No subtitle downloaded")
        raise NotImplementedError

    async def download_sub(self, videos, languages, provider_configs):
        return await self._run_in_thread(
            download_best_subtitles,
            videos, {Language(l)
                     for l in languages},
            provider_configs=provider_configs)

    @staticmethod
    def filter_video_files(files):
        videos = []
        for fil in files:
            _, ext = os.path.splitext(fil)
            if ext in [".jpeg", ".jpg", ".nfo", ".srt", ".sub", ".nbz"]:
                log.debug("Ignoring %s because of extension: %s", fil, ext)
                continue
            videos.append(Video.fromname(fil))
        return videos

    @staticmethod
    def get_subtitle_path(video_file, language):
        return get_subtitle_path(video_file, language=language)

    @staticmethod
    def save_subtitles(video, subtitle_info):
        return save_subtitles(video, subtitle_info)



# -*- coding: utf-8 -*-
"""
test_dopplerr
--------------
Tests for `dopplerr` module.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from pathlib import Path
from unittest import TestCase

from dopplerr.tasks.download_subtitles import DownloadSubtitleTask


class TestDownloader(TestCase):
    @classmethod
    def setup_class(cls):
        pass

    @classmethod
    def teardown_class(cls):
        pass

    def test_sonarr_notify(self):
        pass

    def assert_list_size(self, lst, size):
        if len(lst) != size:
            self.fail("list size should be {}, is {} : {}".format(size, len(lst), ", ".join(lst)))

    def test_glob_simple_filename(self):
        downloader = DownloadSubtitleTask()
        found = downloader.search_file(Path(__file__).parent / "vectors", "videofile.mp4")
        self.assert_list_size(found, 4)
        found = sorted(found)
        self.assertIn("/a_subfolder/prepended-videofile.mp4", found[0])
        self.assertIn("/a_subfolder/videofile-suffixed.mp4", found[1])
        self.assertIn("/a_subfolder/videofile.mp4", found[2])
        self.assertIn("/videofile.mp4", found[3])

    def test_glob_filename_with_bracket(self):
        downloader = DownloadSubtitleTask()
        found = downloader.search_file(Path(__file__).parent / "vectors", "complex[name].mkv")
        self.assert_list_size(found, 2)
        found = sorted(found)
        self.assertIn("vectors/a_subfolder/complex[name].mkv", found[0])
        self.assertIn("vectors/complex[name][withanothersuffix].mkv", found[1])



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import asyncio
import logging

from dopplerr.singleton import singleton
from dopplerr.tasks.download_subtitles import DownloadSubtitleTask

log = logging.getLogger(__name__)


@singleton
class DopplerrTasksManager(object):
    background_tasks = 0

    def post_task(self, task):
        """
        TODO: transfor this simple wrapper arround `create_task ` into a real queue management
        """

        async def wrap_task(task):
            self.background_tasks += 1
            res = await task
            self.background_tasks -= 1
            return res

        loop = asyncio.get_event_loop()
        loop.create_task(wrap_task(task))

    def status(self):
        return {
            'background_tasks': self.background_tasks,
            'subtitle_downloader': {
                'active': 1 if DownloadSubtitleTask().active else 0,
            }
        }
